using System;
using System.Drawing;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Obrancovia
{
    /// <summary>
    /// Mon farby kocky.
    /// </summary>
    public enum FarbaKocky : byte
    {
        Cervena = 0,
        Modra = 1,
        Zelena = 2,
        Zlta = 3,
        SandyBrown = 4,
        WhiteSmoke = 5,
        DarkGoldenRod = 6,
        CornflowerBlue = 7,
        Pink = 8
    }

    /// <summary>
    /// Trieda kocky.
    /// </summary>
    public class Kocka
    {
        #region Kontanty a statick premenn
        public const float MaximumVelocity = 75.0f;
        private const float MaxBoxScaleSpeed = 15.0f;
        private const float MaxBoxScale = 0.01f;
        private const float VelkostKocky = 3.0f;

        // Zoznam farieb pre kocku.
        private static readonly Color[] FarbyKocky = {Color.Red , Color.Blue,
          Color.Green, Color.Yellow, Color.SandyBrown, Color.WhiteSmoke,
          Color.DarkGoldenrod, Color.CornflowerBlue, Color.Pink};

        // Zoznam materilov pre kocku.
        private static Material[] materialyKostky;
        // Sie pre kocku.
        private static Mesh sietKocky = null;
        #endregion

        #region Premenn triedy
        private Vector3 umiestnenie; // Aktulne umiestnenie kocky v rovni.
        private Material material; // Aktulny materil kocky.

        // Mon farby kociek.
        private FarbaKocky[] farby;
        private int indexFarby;
        private bool mozePretekat;
        private float cas = 0.0f;
        private bool maBlikat = false;
        private Vector3 rychlost;
        #endregion

        #region Vytvorenie
        /// <summary>
        /// Vytvor nov instanciu triedy kocky.
        /// </summary>
        /// <param name="zarizeni">D3D zariadenie pre zobrazenie kocky</param>
        public Kocka(Device zariadenie, FarbaKocky[] zoznam, int index,
          bool preteka)
        {
            if ((seznam == null) || (seznam.Length == 0))
                throw new ArgumentNullException("zoznam",
                  "Muste vloi sprvny zoznam.");

            // Vytvorm statick dta.
            if (sietKocky == null)
            {
                VytvorStatickeData(zariadenie);
            }

            // Nastavm zkladn umiestnenie.
            umiestnenie = new Vector3(0, -1.0f, 0);

            // Ulom informcie. 
            farby = zoznam;
            indexFarby = index;
            mozePretekat = preteka;

            // Nastavm zkladn materil.
            NastavFarbuKocky(farby[indexFarby]);
        }

        /// <summary>
        /// Vytvor statick data.
        /// </summary>
        /// <param name="zarizeni">D3D zariadenie na zobrazenie kocky</param>
        private static void VytvorStatickeData(Device zariadenie)
        {
            // Vytvorm sie pre kad vykreslen kocku.
            sietKostky = Mesh.Box(zariadenie, VelkostKocky, VelkostKocky,
              VelkostKocky);
            // Vytvorm zoznam materilov podla farieb pre kocky.
            materialyKocky = new Material[FarbyKocky.Length];
            for (int i = 0; i < materialyKocky.Length; i++)
            {
                materialyKocky[i].Diffuse = materialyKocky[i].Ambient
                  = FarbyKocky[i];
            }
        }

        #endregion

        #region Metody/vlastnosti farby kocky
        /// <summary>
        /// Nastav farbu danej kocky.
        /// </summary>
        /// <param name="barva">Farba kocky</param>
        private void NastavFarbuKocky(FarbaKocky farba)
        {
            material = materialyKocky[(byte)farba];
        }

        /// <summary>
        /// Zska farbu kostky.
        /// </summary>
        public FarbaKocky FarbaKocky
        {
            get { return farny[indexFarby]; }
        }

        /// <summary>
        /// Nastav, i m kocka blika.
        /// </summary>
        public void NastavBlikanieKocky(bool blikat)
        {
            maBlikat = blikat;
        }


        /// <summary>
        /// Nastav aktulny celkov as pre kocku.
        /// </summary>
        public void NastavCasKocky(float celkovyCas)
        {
            cas = celkovyCas;
        }

        /// <summary>
        /// Nastav rchlos kocky po skonen hry.
        /// </summary>
        public void NastavRychlostKocky(Vector3 rychlostKocky)
        {
            rychlost = rychlostKocky;
        }
        public void AktualizujKocku(float uplynuly)
        {
            // Upravme pozciu kocky poda rchlosti.
            Vector3 rychlostSnimkov = rychlost;
            rychlostSnimkov.Normalize();
            rychlostSnimkov.Scale(rychlost.Length() * uplynuly);
            // Posunieme kocku.
            umiestnenie += rychlostSnimkov;
        }

        public void VyberDalsiuFarbu()
        {
            // Sme na konci zoznamu a mu farby preteka? V kladnom prpade sa
            // vrtime na zaiatok zoznamu.
            if ((indexFarby == (farby.Length - 1)) && (mozePretekat))
            {
                // Vrtime sa na zaiatok zoznamu.
                indexFarby = 0;
            }
            // Ak niesme na konci zoznamu, zvime index farby.
            else if (indexFarby != (farby.Length - 1))
            {
                // Zvim index barvy.
                indexFarby++;
            }

            // Nastavme prslun materil.
            NastavFarbuKocky(farby[indexFarby]);
        }
        #endregion

        #region Vlastnosti umiestnenia

        /// <summary>
        /// Aktulne umiestnenie kocky.
        /// </summary>
        public Vector3 Umistnenie
        {
            get { return umistnenie; }
            set { umistnenie = value; }
        }
        #endregion

        #region Upratovanie
        /// <summary>
        /// Upratovanie zdrojov pouitch na zobrazenenie kociek.
        /// </summary>
        public static void Cleanup()
        {
            if (sietKocky != null)
            {
                sietKocky.Dispose();
            }
            sietKocky = null;
            materialyKocky = null;
        }
        #endregion

        #region Zobrazovacia metoda
        /// <summary>
        /// Zobraz kocku.
        /// </summary>
        public void Draw(Device zariadenie)
        {
            // Nastavme materil pre zariadenie na farbu kocky.
            zariadenie.Material = material;
            // Posunieme kocku na sprvnu pozciu.
            if (maBlikat)
            {
                float scaling = (float)Math.Abs(Math.Sin(cas * MaxBoxScaleSpeed));
                scaling *= MaxBoxScale;
                float scaleFactor = 1.0f + scaling;
                zariadenie.Transform.World =
                  Matrix.Translation(umiestnenie) * Matrix.Scaling(
                  scaleFactor, 1.0f, scaleFactor);
            }
            else
            {
                // Posunieme kocku na sprvnu pozciu.
                zariadenie.Transform.World = Matrix.Translation(umiestnenie);
            }
            // Zakeme textry.
            zariadenie.SetTexture(0, null);
            // Vykreslme kocku.
            sietKocky.DrawSubset(0);
        }
        #endregion
    }
}