using System;
using System.Configuration;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Obrancovia
{

    /// <summary>
    /// Trieda obrazoviek uivateskho rozhrania.
    /// </summary>
    public abstract class UiScreen : IDisposable
    {       
        #region Kontanty
        // Kontanty
        public static readonly Vector3 StredObjektu = new Vector3();
        public static readonly int FarbaSpritu = unchecked((int)0xffffffff);

        protected const int SirkaMalehoTlacitka = 128;
        protected const int VyskaMalehoTlacitka = 64;
        protected const int SirkaVelkehoTlacitka = 256;
        protected const int VyskaVelkehoTlacitka = 64;
        #endregion

        #region Instance Data

        protected Sprite zobrazSprite = null;
        protected Texture texturaNaPozadi = null;
        protected Rectangle zdrojPozadi;

        // Rozmery obrazovky
        protected int sirkaObrazovky = 0;
        protected int vyskaObrazovky = 0;

        // Mal by by objekt centrovan?
        private bool jeCentrovany = false;
        protected Vector3 centerUpper;
        #endregion

        #region Vytvorenie
        /// <summary>
        /// Kontruktor objektu obrazovka uivateskho rozhrania.
        /// </summary>
        public UiScreen(Device zariadenie, int sirka, int vyska)
        {
            // Vytvorm nov objekt typu Sprite.
            zobrazSprite = new Sprite(zariadenie);

            // Pre istotu zachytm udalosti zariadenia.
            zariadenie.DeviceLost += new EventHandler(OnDeviceLost);
            zariadenie.DeviceReset += new EventHandler(OnDeviceReset);

            UlozTexturu(null, sirka, vyska, false);
        }

        /// <summary>
        /// Ulo texturu pozadia.
        /// </summary>
        protected void UlozTexturu(Texture pozadia, int sirka, int vyska,
        bool centrovatTexturu)
        {
            // Ulom texturu pozadia.
            texturaNaPozadi = pozadia;

            if (texturaNaPozadi != null)
            {
                // Zskam textru pozadia.
                using (Surface s = texturaNaPozadi.GetSurfaceLevel(0))
                {
                    SurfaceDescription desc = s.Description;
                    zdrojPozadia = new Rectangle(0, 0,
                    desc.Width, desc.Height);
                }
            }

            // Ulom ku a vku.
            sirkaObrazovky = sirka;
            vyskaObrazovky = vyska;

            // Ulom stav centrovania textry.
            jeCentrovany = centrovatTexturu;

            if (jeCentrovany)
            {
                centerUpper = new Vector3(
                (float)(sirka - zdrojPozadia.Width) / 2.0f,
                (float)(vyska - zdrojPozadia.Height) / 2.0f, 0.0f);
            }
        }
        #endregion

        #region Udalosti zariadenie
        private void OnDeviceLost(object sender, EventArgs e)
        {
            // Zariadenie bolo straten take, musm uprata vytvoren sprit.
            if (zobrazSprite != null)
                zobrazSprite.OnLostDevice();
        }

        private void OnDeviceReset(object sender, EventArgs e)
        {
            // Zariadenie bolo resetovan, musm opravi vytvoren sprit.
            if (zobrazSprite != null)
                zobrazSprite.OnResetDevice();
        }
        #endregion

        #region Zobrazenie
        /// <summary>
        /// Zanm kresli s danm spritom.
        /// </summary>
        protected void ZaciatokSprite()
        {
            zobrazSprite.Begin(SpriteFlags.AlphaBlend);
        }
        /// <summary>
        /// Koniec kreslenia s danm spritom.
        /// </summary>
        protected void KonecSprite()
        {
            zobrazSprite.End();
        }
        /// <summary>
        /// Zobraz vytvoren obrazovku.
        /// </summary>
        public virtual void Draw()
        {
            // Zobrazm pozadie, ak existuje.
            if (texturaNaPozadi != null)
            {
                if (jeCentrovany)
                {
                    // Zobrazm texturu na stred.
                    zobrazSprite.Draw(texturaNaPozadi, zdrojPozadia,
                    StredObjektu, centerUpper, FarbaSpritu);
                }
                else
                {
                    // Nastavm sprvnu vekost pozadia.
                    zobrazSprite.Transform = Matrix.Scaling(
                      (float)sirkaObrazovky / (float)zdrojPozadia.Width,
                      (float)vyskaObrazovky / (float)zdrojPozadia.Height,
                      0);

                    // Zobrazm sprit na obrazovku.
                    zobrazSprite.Draw(texturaNaPozadi, zdrojPozadia,
                      StredObjektu, StredObjektu, FarbaSpritu);
                    // Zrum transformciu.
                    zobrazSprite.Transform = Matrix.Identity;
                }
            }
        }
        #endregion

        #region IDisposable Members
        /// <summary>
        /// Upracem objekt, ak neni volan Dispose.
        /// </summary>
        ~UiScreen()
        {
            Dispose();
        }
        /// <summary>
        /// Upracem pouit zdroje.
        /// </summary>
        public virtual void Dispose()
        {
            GC.SuppressFinalize(this);
            // Upracem sprit.
            if (zobrazSprite != null)
            {
                zobrazSprite.Dispose();
            }
            zobrazSprite = null;
        }

        #endregion
    }
    /// <summary>
    /// Trieda pre tlatko, ktor budem zobrazova pomocou DirectX.
    /// </summary>
    public class UiButton
    {
        #region Premenne triedy
        private Sprite zobrazSprite = null;
        private Texture texturaTlacitka = null;
        private Texture texturaStisknutehoTlacitka = null;
        private Rectangle zdrojStisknuteho;
        private Rectangle zdroj;
        private Vector3 umiestnenie;
        private bool jeTlacitkoStisknuto = false;
        private Rectangle oblastTlacitka;

        // Udalos kliknutia
        public event EventHandler Click;

    #endregion

        #region Vytvorenie

        /// <summary>
        /// Vytvor nov instanciu triedy tlatka s pouitm existujceho spritu.
        /// </summary>
        public UiButton(Sprite sprite, Texture stisknute, Texture nestisknute,
          Rectangle rectStisknute, Rectangle rectNestisknute,
          Point umiestnenieTlacitka)
        {
            // Ulom objekt typu Sprite.
            zobrazSprite = sprite;

            // Ulom textury.
            texturaTlacitka = nestisknute;
            texturaStisknutehoTlacitka = stisknute;

            // Obdniky
            zdrojStisknuteho = rectStisknute;
            zdroj = rectNestisknute;

            // Umiestnenie
            umiestnenie = new Vector3(umiestnenieTlacitka.X, umiestnenieTlacitka.Y, 0);

            // Vytvorm obdik poda umiestnenia a rozmerov tlatka.
            oblastTlacitka = new Rectangle((int)umiestnenie.X, (int)umiestnenie.Y,
              zdrojStisknuteho.Width, zdrojStisknuteho.Height);
        }
        #endregion

        #region Udalosti myi
        /// <summary>
        /// Aktualizuje tlatko, ak nad nm prejdem ukazovateom myi.
        /// </summary>
        public void OnMouseMove(int x, int y)
        {
            // Rozhodnem, i je tlatko stisnknut.
            jeTlacitkoStisknute = oblastTlacitka.Contains(x, y);
        }
        /// <summary>
        /// Rozpozn kliknutie na tlatko.
        /// </summary>
        public void OnMouseClick(int x, int y)
        {
            // Rozhodnem, i uvate klikol na tlatko.
            if (oblastTlacitka.Contains(x, y))
            {
                if (Click != null)
                    Click(this, EventArgs.Empty);
            }
        }
        #endregion

        #region Zobrazenie
        /// <summary>
        /// Zobraz tlatko.
        /// </summary>
        public void Draw()
        {
            if (jeTlacitkoStisknute)
            {
                zobrazSprite.Draw(texturaStisknutehoTlacitka, zdrojStisknuteho,
                  UiScreen.StredObjektu, umiestnenie, UiScreen.FarbaSpritu);
            }
            else
            {
                zobrazSprite.Draw(texturaTlacitka, zdroj, UiScreen.StredObjektu,
                  umiestnenie, UiScreen.FarbaSpritu);
            }
        }
        #endregion
    }
}